/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Send Tab URLs.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2004-2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


var gSendTabURLs = {
  // List style constants
  FMTOPT_NO_FORMATTING:  0,
  FMTOPT_NUMBERED:       1,
  FMTOPT_BULLETED:       2,

  // Send to constants
  SENDTO_CLIPBOARD: 1,
  SENDTO_GMAIL: 2,
  SENDTO_GOOGLEAPPS: 3,
  SENDTO_YAHOOMAIL: 4,

  _utils:   null,
  _strBundle:  null,


  // Method handleEvent() effectively makes gSendTabURLs an implementation of
  // the EventListener interface; therefore it can be passed as the listener
  // argument to window.addEventListener() and window.removeEventListener()

  handleEvent: function (aEvent)
  {
    // When this method is invoked, 'this' will not refer to gSendTabURLs.
    // The 'that' object will refer to 'this'. =)
    var that = gSendTabURLs;

    if (aEvent.type == "load") {
      that.initSendTabs();
    }
    else if (aEvent.type == "unload") {
      window.removeEventListener("load", that, false);
      window.removeEventListener("unload", that, false);
    }
  },


  initSendTabs: function ()
  {
    this._utils  = new aeSendTabsUtils();
    this._strBundle = document.getElementById("ae-sendtabs-strings");

    var appName = this._strBundle.getString("appName");
    this._utils.log(appName + " initialized; host OS is " + this._utils.getOS());

    var isYahooMailEnabled = this._utils.getPref("sendtabs.mailclient.yahoomail.enabled", false);
    if (isYahooMailEnabled) {
      this._utils.log("sendtaburls: Yahoo! Mail is enabled.");
      var ymailUrlPrefix = this._utils.getPref("sendtabs.mailclient.yahoomail.mail_url");
      if (! ymailUrlPrefix) {
	this._utils.log("sendtaburls: Yahoo! Mail compose URL prefix is not set");
	ymailUrlPrefix = this._utils.getPref("yahoo_options");
	if (ymailUrlPrefix) {
	  this._utils.log("sendtaburls: Found Yahoo! Mail compose URL prefix from Email This!:\n" + ymailUrlPrefix);
	  this._utils.setPref("sendtabs.mailclient.yahoomail.mail_url",
			      ymailUrlPrefix);
	  this._utils.log("sendtaburls: Setting Yahoo! Mail compose URL prefix from Email This! configuration.");
	}
      }
    }
  },

  
  sendTabs: function ()
  {
    var isFormatPromptEnabled = this._utils.getPref("sendtabs.message.format.always_ask");
    if (isFormatPromptEnabled) {
      var dlgArgs = { cancel: null };
      window.openDialog("chrome://sendtabs/content/formatOpts.xul", "ae_sendtabs_formatopts", "modal,centerscreen", dlgArgs);
      if (dlgArgs.cancel) {
	return;
      }
    }

    const LF = "\r\n";
    var mailDest = this._utils.getPref("sendtabs.mailclient");
    var tabbrowser = document.getElementById("content");
    var numTabs = tabbrowser.browsers.length;
    var maxTabs = mailDest == this.SENDTO_CLIPBOARD ? numTabs : 30;
    var body = "";
    var listStyle = this._utils.getPref("sendtabs.message.format.list_style");
    var listChar = "";

    if (listStyle == this.FMTOPT_BULLETED) {
      listChar = "- ";
    }

    for (var i = 0; i < numTabs && i < maxTabs; i++) {
      if (listStyle == this.FMTOPT_NUMBERED) {
	listChar = i + 1 + ". ";
      }

      var title = "";
      var selection = tabbrowser.browsers[i].contentWindow.getSelection();
      if (selection != "") {
	title = selection;
      }
      else {
	title = tabbrowser.browsers[i].contentDocument.title;
      }

      if (title == "") {
	var hdgs = tabbrowser.browsers[i].contentDocument.getElementsByTagName("h1");
	if (hdgs && hdgs.length > 0) {
	  title = hdgs[0].textContent;
	}
	else {
	  title = this._strBundle.getString("untitledPage");
	}
      }

      body += listChar + title + LF 
 	    + tabbrowser.browsers[i].currentURI.spec + LF + LF;
    }

    var totalTabs = numTabs > maxTabs ? maxTabs : numTabs;  //!! Fix for clpbd
    var subj = this._strBundle.getFormattedString("subjectLine", [totalTabs]);
    var mailStr;  
    var newTab;

    switch (mailDest) {
    case this.SENDTO_CLIPBOARD:  // Clipboard
      mailStr = body;
      var clipbd = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
	                     .getService(Components.interfaces
					 .nsIClipboardHelper);
      clipbd.copyString(mailStr);
      window.setTimeout(this._showCopyToClipboardConfirmation, 200);
      return;

    case this.SENDTO_GMAIL:  // Gmail
      mailStr = "https://mail.google.com/mail/?ui=2&view=cm&tf=0&to=&su=" 
	        + encodeURIComponent(subj) + "&body="
	        + encodeURIComponent(body) + "";
      this._utils.log("Gmail compose URL:\n\n" + mailStr);
      this._loadWebMailClient(mailStr);
      return;
      
    case this.SENDTO_GOOGLEAPPS:
      var gappsDomain = this._utils.getPref("sendtabs.mailclient.googleapps.domain");
      if (! gappsDomain) {
	var dlgArgs = {};
	window.openDialog("chrome://sendtabs/content/googleApps.xul",
			  "dlg_sendtabs_googapps",
			  "chrome,modal,centerscreen", dlgArgs);
      }

      if ('googleAppsDomain' in dlgArgs) {
	gappsDomain = dlgArgs.googleAppsDomain;
	this._utils.log("sendtabs: Google Apps domain name: " + gappsDomain);
      }
      else {
	this._utils.log("sendtabs: Google Apps domain name was not set.");
	return;
      }

      if (gappsDomain) {
	this._utils.setPref("sendtabs.mailclient.googleapps.domain",
			    gappsDomain);
	mailStr = "https://mail.google.com/a/" + gappsDomain
	          + "/?ui=2&view=cm&tf=0&to=&su=" + encodeURIComponent(subj) 
	          + "&body=" + encodeURIComponent(body) + "";
	this._utils.log("Google Apps Gmail compose URL:\n\n" + mailStr);
	this._loadWebMailClient(mailStr);
      }

      return;

    case this.SENDTO_YAHOOMAIL:  // Yahoo! Mail (classic)
      // The Yahoo! Mail compose URL prefix needs to be specified by the user;
      // see <http://www.lazyrussian.com/my-projects/email-this-firefox-extension/>
      // WARNING: Doesn't work well with Rich Text mode in Yahoo! Mail.
      var ymailURL = this._utils.getPref("sendtabs.mailclient.yahoomail.mail_url");
      if (! ymailURL) {
	this._utils.alertEx(this._strBundle.getString("appName"),
			    this._strBundle.getString("errAppExecError"));
	return;
      }

      mailStr = ymailURL + "compose?&To=&Subj=" + encodeURIComponent(subj) 
	        + "&Body=" + encodeURIComponent(body) + "";
      this._utils.log("Yahoo! Mail compose URL:\n\n" + mailStr);
      this._loadWebMailClient(mailStr);
      return;

    default:
      break;
    }

    // Workaround to "mailto" URL length constraint (~462 chars) on Windows.
    // Execute supported apps (Thunderbird, SeaMonkey, Outlook Express, etc.)
    // with command line parameters to create email message.
    // All other email clients: just pass the "mailto" URL, taking a risk at
    // not being able to start the default email client at all.
    if (this._utils.getOS() == "WINNT") {
      try {
	var result = this._winExecEmailApp(subj, body);
      }
      catch (e) {
	var msg = this._strBundle.getString("errAppExecError");
	msg += (this._utils.DEBUG ? "\n\n".concat(e) : "");
	this._utils.alertEx(this._strBundle.getString("appName"), msg);
	return;
      }

      if (! result) {
	this._utils.log(this._strBundle.getString("appName") + " cannot find a supported email app; will attempt to launch 'mailto:' URL");
	this._execMailtoURL(subj, body);
      }
    }
    else {
      this._execMailtoURL(subj, body);
    }
  },


  _showCopyToClipboardConfirmation: function ()
  {
    window.openDialog("chrome://sendtabs/content/copyToClipboard.xul",
		      "dlg_sendtabs_copytoclpbd", "chrome,modal,centerscreen");
  },


  _loadWebMailClient: function (aMailStr)
  {
    var openInNewWnd = this._utils.getPref("sendtabs.mailclient.open_in_new_window", false);
    if (openInNewWnd) {
      window.open(aMailStr);
    }
    else {
      var newTab = gBrowser.loadOneTab(aMailStr);
      gBrowser.selectedTab = newTab;
    }
  },


  /*
   * Throws exception if app execution failed, otherwise returns true if email
   * app successfully executed, false if a supported email app wasn't found.
   */
  _winExecEmailApp: function (aSubject, aBody)
  {
    var rv = false;
    var cmdLine = this._getWindowsDefaultEmailAppCmdLine();
    if (! cmdLine) {
      return rv;
    }

    cmdLine = this._sanitizeCmdLineStr(cmdLine);
    cmdLine = this._winSubstCmdLineVars(cmdLine);
    cmdLine = cmdLine.substring(0, (cmdLine.toLowerCase().indexOf(".exe") + 4));
    this._utils.log("Command line of default email app:\n\"" + cmdLine + "\"");
    
    var subj = encodeURIComponent(aSubject);
    var body = encodeURIComponent(aBody);
    var mailto = "mailto:?subject=" + subj + "&body=" + body;
    var mailApp = new aeWindowsApp();
    mailApp.setPath(cmdLine);

    // Check for supported email apps
    // Thunderbird, Seamonkey, Mozilla suite, or Netscape 7
    if (cmdLine.search(/thunde(rbird)?/i)  != -1
	|| cmdLine.search(/seamon(key)?/i) != -1
	|| cmdLine.search(/mozilla\.exe/i) != -1  // Mozilla 1.x suite
	|| cmdLine.search(/netscp\.exe/i)  != -1  // Netscape 7
	// NOTE: Netscape versions older than v.7 are not supported
	) {

      mailApp.setArgs("-compose", "to=,subject=" + subj + ",body=" + body);
    }
    // Outlook Express, Windows Mail or Windows Live Mail
    else if (cmdLine.search(/msimn\.exe/i) != -1
	     || cmdLine.search(/winmail\.exe/i) != -1
	     || cmdLine.search(/wlmail\.exe/i)  != -1) {
      mailApp.setArgs("/mailurl:" + mailto);
    }
    // MS Office Outlook
    else if (cmdLine.search(/outlook\.exe/i) != -1) {
      mailApp.setArgs("/c", "ipm.note", "/m", mailto);
    }
    // Eudora
    else if (cmdLine.search(/eudora\.exe/i) != -1) {
      // !! BUG !!
      // Eudora won't take long mailto: URLs.  Eudora main window will open,
      // but no compose window will appear.
      return rv;
    }
    else {
      return rv;
    }

    try {
      mailApp.exec();
    }
    catch (e) {
      throw e;
    }
    rv = true;

    return rv;
  },

  
  _getWindowsDefaultEmailAppCmdLine: function () 
  {
    var rv, cmdLine;
    var wrk = Components.classes["@mozilla.org/windows-registry-key;1"]
	                .createInstance(Components.interfaces.nsIWindowsRegKey);
    // Determine default mail app from Windows registry
    try {
      wrk.open(wrk.ROOT_KEY_CLASSES_ROOT, "mailto\\shell\\open\\command",
	       wrk.ACCESS_READ);
      cmdLine = wrk.readStringValue("");  // Get default value
    }
    catch (e) {
      // Reach here if the above registry key wasn't defined
      this._utils.log(e);
    }
    finally {
      wrk.close();
    }

    rv = cmdLine;
    return rv;
  },


  _execMailtoURL: function (aSubject, aBody)
  {
    var uri;
    var mailtoURL = "mailto:?subject=" + encodeURIComponent(aSubject) + "&body=" + encodeURIComponent(aBody);
    this._utils.log("Length of mailto URL: " + mailtoURL.length + " chars");

    var ioSvc = Components.classes["@mozilla.org/network/io-service;1"]
                          .getService(Components.interfaces.nsIIOService);

    uri = ioSvc.newURI(mailtoURL, null, null);

    var eps = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"].getService(Components.interfaces.nsIExternalProtocolService);

    try {
      // !!! BUG !!!
      // Windows won't accept mailto: URLs longer than ~462 char's
      eps.loadUrl(uri);
    } 
    catch (e) { 
      this._utils.alertEx(this._strBundle.getString("appName"),
			  this._strBundle.getString("errAppExecError"));
      this._utils.log(e); 
    }
  },


  /*
   * Removes quotation marks from the given string.
   */
  _sanitizeCmdLineStr: function (aString)
  {
    return aString.replace(/\"/g, "");
  },


  /*
   * Substitues the following Windows environment variables with actual paths:
   * %ProgramFiles%, %windir%, %systemroot%
   * (capitalization of environment variables do not matter)
   */
  _winSubstCmdLineVars: function (aString)
  {
    var dirProp = Components.classes["@mozilla.org/file/directory_service;1"]
                            .getService(Components.interfaces.nsIProperties);
    var progFiles = dirProp.get("ProgF", Components.interfaces.nsIFile);
    var winDir = dirProp.get("WinD", Components.interfaces.nsIFile);

    var str = aString.replace(/\%ProgramFiles\%/i, progFiles.path);
    str = str.replace(/(\%windir\%)|(\%systemroot\%)/i, winDir.path);

    return str;
  }
};


//
// Event handler initialization
//

window.addEventListener("load",   gSendTabURLs, false);
window.addEventListener("unload", gSendTabURLs, false);
